#ifndef FRAMEBUFFER_H
#define FRAMEBUFFER_H

#include "glew.h"
#include <vector>

class Texture;

class FrameBuffer
{
protected:
    /**
     * @brief check() checks the state of the framebuffer and returns false if it isn't ready for use
     */
    bool check();

    const bool allocated;
    GLuint fbo;
    std::vector<Texture*> textures;
    std::vector<GLuint> attachments;

public:
    /**
     * @brief this constructor should be used to create a framebuffer, once created,
     * use addTexture to attach textures to the framebuffer, then use initColorAttachments to
     * effectively bind them to the Framebuffer, at this point, the framebuffer should be ready to use.
     */
    FrameBuffer();

    /**
     * @brief this constructor is only used to get a handle on an existing Framebuffer created by another library
     */
    FrameBuffer(GLuint id) : allocated(false), fbo(id) {}

    /**
     * @brief the destructor does not destroy the textures, however, deleteTextures will do that
     */
    ~FrameBuffer();

    void addTexture(Texture* tex, GLenum attachment);

    void initColorAttachments();

    void deleteTextures();

    /**
     * @brief bindFBO binds the framebuffer on the specified target
     */
    void bindFBO(GLenum target = GL_FRAMEBUFFER) const;

    /**
     * @brief getTexture returns an attached texture, indexed from their adding order.
     */
    Texture* getTexture(int texId);

    /**
     * @brief screen is a Framebuffer handle on the screen
     */
    static const FrameBuffer* screen;
};

#endif // FRAMEBUFFER_H
