#include "scene.h"
#include "mesh.h"
#include <set>


std::vector<unsigned int> Scene::getMeshTypes()
{
    std::set<unsigned int> typesSet;
    std::vector<unsigned int> types;
    for(SceneIterator<GeometryNode*>* geometryIt = scene->getGeometry();
        geometryIt->isValid(); geometryIt->next())
    {
        typesSet.emplace(geometryIt->getItem()->mesh->getFlags());
    }
    for(unsigned int type : typesSet)
        types.push_back(type);
    return types;
}

std::vector<unsigned int> Scene::getLightTypes()
{
    std::set<unsigned int> typesSet;
    std::vector<unsigned int> types;
    for(SceneIterator<Light*>* lightIt = scene->getLights();
        lightIt->isValid(); lightIt->next())
    {
        typesSet.emplace(getFlags(geometryIt->getItem()));
    }
    for(unsigned int type : typesSet)
        types.push_back(type);
    return types;
}

BasicScene::~BasicScene()
{
    clearScene();
}

void BasicScene::addMesh(GeometryNode* gn)
{
    geometry->push_back(gn);
}

GeometryNode* BasicScene::addMesh(Mesh* m, glm::mat4 transform)
{
    GeometryNode *gn = new GeometryNode(m, transform);
    allocations.push_back(gn);
    addMesh(gn);
    return gn;
}

void BasicScene::addLight(Light* myLight)
{
    lights.push_back(myLight);
}

void BasicScene::clearScene()
{
    for(GeometryNode *gn : allocations)
        delete gn;
    allocations.clear();
    geometry.clear();
    lights.clear();
}

SceneIterator<Light*>* BasicScene::getLights()
{
    return new ArrayIterator<Light*>(lights);
}

SceneIterator<GeometryNode*>* BasicScene::getGeometry()
{
    return new ArrayIterator<GeometryNode*>(geometry);
}

